
#if	!defined(PTR)
#error missing macro-constant "PTR".
#elif	PTR<0 || PTR>2
#error bad value of macro-constant "PTR".
#endif

#if	!defined(TSH)
#error missing macro-constant "TSH".
#elif	TSH<0 || TSH>5
#error bad value of macro-constant "TSH".
#endif

#include <stdio.h>
#include <stdlib.h>

#if	PTR==0
#include <math.h>	/* for log() */
#endif

#include "rif_f.h"
#include "csi_f.h"

extern void	Error(char *msg);

static void	Write(double i0,double i,FILE *file)
{
#if	PTR==0
	double	o=(i0<=0.0 || i<=0.0)?0.0:log(i0/i);
#elif	PTR==1
	double	o=(i0<=0.0)?0.0:i/i0;
#elif	PTR==2
	double	o=i-i0;
#endif

	(void)fwrite(&o,sizeof(double),1,file);
}

int	main(int argc,char **argv)
{
	char	**path;
	int	Nx,Ny,Nz,Y,z,x,y;
	FILE	*file;
	FOM	**I;

#if	PTR==0 || PTR==1
#define PREFIX	"i0e"
#define RAW	"RI/"
#elif	PTR==2
#define PREFIX	"r0e"
#define RAW	"RR/"
#endif

#if	TSH==0
	int	Nt;

#define ARGC	6
#define SUFFIX	"t"
#define MARGIN	"top"
#elif	TSH==1
	int	Nl,Nr,LR;

#define ARGC	7
#define SUFFIX	"s"
#define MARGIN	"left right"
#else
	int	Nt,Nl,Nr,LR;
	double	*T;

#define ARGC	8
#if	TSH==2
#define SUFFIX	"h_0"
#elif	TSH==3
#define SUFFIX	"h_1"
#elif	TSH==4
#define SUFFIX	"h_2"
#elif	TSH==5
#define SUFFIX	"h_3"
#endif
#define MARGIN	"top left right"
#endif

	if (argc!=ARGC)
	Error("usage : " PREFIX "_" SUFFIX " " RAW " NF " MARGIN " layer SG");

	path=CheckSliceImages_Float(argv[1],argv[2],&Nx,&Ny,&Nz,NULL);

	if (Nz<2) Error("too few views.");

	free(path[--Nz]);

	if ((Y=atoi(argv[argc-2]))<0 || Y>=Ny) Error("bad sinogram layer.");

#if	TSH==0
	if ((Nt=atoi(argv[3]))<=0 || Nt>=Ny) Error("bad top margin.");
#elif	TSH==1
	if ((Nl=atoi(argv[3]))<0 ||
	    (Nr=atoi(argv[4]))<0 || (LR=Nl+Nr)<=0 || LR>=Nx)
	    Error("bad left and/or right margin.");
#elif	TSH>=2 && TSH<=5
	if ((Nt=atoi(argv[3]))<=0 || Nt>=Ny ||
	    (Nl=atoi(argv[4]))<0 ||
	    (Nr=atoi(argv[5]))<0 || (LR=Nl+Nr)<(int)(TSH/2) || LR>=Nx)
	    Error("bad top, left and/or right margin.");

	if ((T=(double *)malloc(sizeof(double)*Nx))==NULL)
	    Error("memory allocation error.");
#endif

	if (argv[argc-1][0]=='-')
	    file=stdout;
	else
	    if ((file=fopen(argv[argc-1],"wb"))==NULL) Error("file not open.");

	(void)fprintf(file,"%d\t%d\n",Nx,Nz);

	for (z=0; z<Nz; z++) {
	    (void)ReadImageFile_Float(path[z],&x,&y,&I,NULL);
	    if (x!=Nx || y!=Ny) Error("image size not match.");
{
	FOM	*i=I[Y];

#if	TSH==0
	double	t;

	for (x=0; x<Nx; x++) {
	    t=0.0; for (y=0; y<Nt; y++) t+=I[y][x];

	    Write(t/(double)Nt,*(i++),file);
	}
#elif	TSH==1
	double	a,b,l=0.0,r=0.0;

	for (x=0    ; x<Nl; x++) l+=i[x];
	for (x=Nx-Nr; x<Nx; x++) r+=i[x];

	if (Nl==0 || Nr==0) {
	    a=0.0; b=(l+r)/(double)LR;
	}
	else {
	    l/=(double)Nl; r/=(double)Nr;
	    a=(r-l)/((double)Nx-(double)LR/2.0); b=l-a*(double)(Nl-1)/2.0;
	}
	for (x=0; x<Nx; x++) Write(a*(double)x+b,*(i++),file);
#elif	TSH==2
	double	t,c=0.0;

	for (x=0; x<Nx; x++) {
	    t=0; for (y=0; y<Nt; y++) t+=I[y][x]; T[x]=t;

	    if (x<Nl || x>=Nx-Nr) c+=(i[x]/t);
	}
	c/=(double)LR; for (x=0; x<Nx; x++) Write(c*T[x],*(i++),file);
#elif	TSH==3
	double	t,c,s1=0.0,s2=0.0;

	for (x=0; x<Nx; x++) {
	    t=0; for (y=0; y<Nt; y++) t+=I[y][x]; T[x]=t;

	    if (x<Nl || x>=Nx-Nr) {
		s1+=(t*t); s2+=(i[x]*t);
	    }
	}
	c=s2/s1; for (x=0; x<Nx; x++) Write(c*T[x],*(i++),file);
#elif	TSH==4
	double	t,X,it,d,a,b,s1=0.0,s2=0.0,s3=0.0,s4=0.0;

	for (x=0; x<Nx; x++) {
	    t=0; for (y=0; y<Nt; y++) t+=I[y][x]; T[x]=t;

	    if (x<Nl || x>=Nx-Nr) {
		X=(double)x; it=i[x]/t; s1+=(X*X); s2+=X; s3+=(X*it); s4+=it;
	    }
	}
	d=s1*(double)LR-s2*s2; a=((double)LR*s3-s2*s4)/d; b=(s1*s4-s2*s3)/d;
	for (x=0; x<Nx; x++) Write((a*(double)x+b)*T[x],*(i++),file);
#elif	TSH==5
	double	t,X,X2,t2,it,d,a,b,s1=0.0,s2=0.0,s3=0.0,s4=0.0,s5=0.0;

	for (x=0; x<Nx; x++) {
	    t=0; for (y=0; y<Nt; y++) t+=I[y][x]; T[x]=t;

	    if (x<Nl || x>=Nx-Nr) {
		X=(double)x; X2=X*X; t2=t*t; it=i[x]*t;
		s1+=(X2*t2); s2+=(X*t2); s3+=t2; s4+=(X*it); s5+=it;
	    }
	}
	d=s1*s3-s2*s2; a=(s3*s4-s2*s5)/d; b=(s1*s5-s2*s4)/d;
	for (x=0; x<Nx; x++) Write((a*(double)x+b)*T[x],*(i++),file);
#endif
}
	    for (y=0; y<Ny; y++) free(I[y]); free(I); free(path[z]);
	}
	if (file!=stdout) (void)fclose(file);

#if	TSH>=2 && TSH<=5
	free(T);
#endif

	free(path); 

	return 0;
}
