
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include "rif_f.h"
#include "csi_f.h"

#define PATH_LEN	2048

static void	Error(char *head,char *tail)
{
	(void)fputs(head,stderr); (void)fputs(" : ",stderr);
	(void)fputs(tail,stderr); (void)fputs(".\n",stderr); exit(1);
}

#define ALLOC(type,size)	(type *)malloc(sizeof(type)*(size_t)(size))

static char	Directory[PATH_LEN];

static char	**Alloc(int Nz)
{
	char	**paths=ALLOC(char *,Nz);

	if (paths==NULL) Error(Directory,"no memory for paths");

	return paths;
}

static int	DirLen,nx,ny,bps;

static char	*Peep(char *name)
{
	char	*path=ALLOC(char,DirLen+strlen(name)+1);
	int	Nx,Ny,BPS;

	if (path==NULL) Error(Directory,"no memory for path");

	(void)strcat(strcpy(path,Directory),name);
	BPS=ReadImageFile_Float(path,&Nx,&Ny,NULL,NULL);

	if (nx<Nx) nx=Nx;
	if (ny<Ny) ny=Ny; if (bps<BPS) bps=BPS; return path;
}

static int	Normal(char **path1,char **path2)
{
	return strcmp(*path1+DirLen,*path2+DirLen);
}

static int	Reverse(char **path1,char **path2)
{
	return Normal(path2,path1);
}

static int	Numeric(char *path,double *d)
{
	for (path+=DirLen; *path!='\0'; path++)
	    if (*path=='-' || *path=='+' ||
	       (*path>='0' && *path<='9')) return sscanf(path,"%lf",d);

	return 0;
}

static int	Ascending(char **path1,char **path2)
{
	double	d1,d2;
	int	i1=Numeric(*path1,&d1),
		i2=Numeric(*path2,&d2);

	return (i1==1 && i2==1)?(d1>d2)?1:(d1<d2)?-1:0:
	       (i1==1)?1:(i2==1)?-1:Normal(path1,path2);
}

static int	Descending(char **path1,char **path2)
{
	return Ascending(path2,path1);
}

char	**CheckSliceImages_Float(char *directory,
				 char *nameFile,
				 int *Nx,
				 int *Ny,
				 int *Nz,
				 int *BPS)
{
	int	nz,z;
	char	name[PATH_LEN],
		*nifn="no image file name",
		*mifn="missing image file name";
	char	**paths;

	DirLen=strlen(strcat(strcpy(Directory,directory),"/")); nx=ny=bps=0;

	if (nameFile[0]!='-')
{
	FILE	*file;
	char	line[PATH_LEN],
		*tll="too long line";

	if ((file=fopen(nameFile,"r"))==NULL)
	    Error(nameFile,"name file not open");

	for (nz=0; fgets(line,PATH_LEN,file)!=NULL; )
	    if (*line!='\0') {
		if (line[strlen(line)-1]!='\n') Error(nameFile,tll);

		if (sscanf(line,"%s",name)==1) ++nz;
	    }

	if (nz==0) Error(nameFile,nifn);

	paths=Alloc(nz);

	rewind(file);

	for (z=0; z<nz && fgets(line,PATH_LEN,file)!=NULL; )
	    if (*line!='\0') {
		if (line[strlen(line)-1]!='\n') Error(nameFile,tll);

		if (sscanf(line,"%s",name)==1) paths[z++]=Peep(name);
	    }

	if (z!=nz) Error(nameFile,mifn);

	(void)fclose(file);
}
	else if (nameFile[1]!='\0' && nameFile[1]!='-')
{
	char	*ptr;

	ptr=nameFile+1;
	for (nz=0; sscanf(ptr,"%s",name)==1; nz++)
	    ptr=strstr(ptr,name)+strlen(name);

	if (nz==0) Error(nameFile+1,nifn);

	paths=Alloc(nz);

	ptr=nameFile+1;
	for (z=0; z<nz && sscanf(ptr,"%s",name)==1; z++) {
	    paths[z]=Peep(name);
	    ptr=strstr(ptr,name)+strlen(name);
	}
	if (z!=nz) Error(nameFile+1,mifn);
}
	else
{
	DIR		*dir;
	struct dirent	*ent;

	if ((dir=opendir(directory))==NULL)
	    Error(directory,"directory not open");

	for (nz=0; (ent=readdir(dir))!=NULL; )
	    if (strcmp(ent->d_name,".") && strcmp(ent->d_name,"..")) ++nz;

	if (nz==0) Error(directory,nifn);

	paths=Alloc(nz);

	rewinddir(dir);

	for (z=0; z<nz && (ent=readdir(dir))!=NULL; )
	    if (strcmp(ent->d_name,".") && strcmp(ent->d_name,".."))
		paths[z++]=Peep(ent->d_name);

	if (z!=nz) Error(directory,mifn);

	(void)closedir(dir);

	if (nameFile[1]=='\0')
	    qsort(paths,(size_t)nz,sizeof(*paths),(int (*)())Normal);
	else if (nameFile[2]=='r' && nameFile[3]=='\0')
	    qsort(paths,(size_t)nz,sizeof(*paths),(int (*)())Reverse);
	else if (nameFile[2]=='n' && nameFile[3]=='\0')
	    qsort(paths,(size_t)nz,sizeof(*paths),(int (*)())Ascending);
	else if ((nameFile[2]=='r' && nameFile[3]=='n') ||
		 (nameFile[2]=='n' && nameFile[3]=='r'))
	    qsort(paths,(size_t)nz,sizeof(*paths),(int (*)())Descending);
	else
	    Error(nameFile+1,"unknown sorting option");
}
#ifdef	CSI_SLICE
	for (z=0; z<nz; z++) paths[z]+=DirLen;
#endif
	if (Nx!=NULL) *Nx=nx;
	if (Ny!=NULL) *Ny=ny;
	if (Nz!=NULL) *Nz=nz; if (BPS!=NULL) *BPS=bps; return paths;
}
